function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { useContext, useState, useEffect, useCallback, useImperativeHandle } from 'react';
import { RouterContext, RouterViewContext } from '../context';
import { innumerable, isFunction, isPlainObject, readRouteMeta } from '../util';

function isCommonPage(matched, commonPageName) {
  return Boolean(commonPageName && matched.some(r => readRouteMeta(r.config, commonPageName)));
}

function getRouteMatched(router, currentRoute, commonPageName) {
  if (!currentRoute || !router) return [];
  let {
    matched = []
  } = currentRoute;

  if (matched) {
    const _isCommonPage = isCommonPage(matched, commonPageName);

    if (_isCommonPage && currentRoute.query.redirect) {
      matched = router.getMatched(currentRoute.query.redirect);
    }
  }

  return matched;
}

function useRouter(defaultRouter) {
  // eslint-disable-next-line react-hooks/rules-of-hooks
  return defaultRouter || useContext(RouterContext);
}

function useRoute(defaultRouter, options = {}, anotherWatch = null) {
  const router = useRouter(defaultRouter);
  const [route, setRoute] = useState(router ? router.currentRoute || router.initialRoute : null);

  if (options.watch && router) {
    // eslint-disable-next-line react-hooks/rules-of-hooks
    const onRouteChange = useCallback(async (route, prevRoute, router) => {
      if (options.ignoreSamePath !== false && route && prevRoute && route.fullPath === prevRoute.fullPath) return;
      if (anotherWatch && (await anotherWatch(route, prevRoute, router)) === false) return;
      if (isFunction(options.watch) && (await options.watch(route, prevRoute, router)) === false) return;
      if (options.delay) setTimeout(() => setRoute(route), typeof options.delay === 'number' ? options.delay : 0);else setRoute(route);
    }, [options, anotherWatch]); // eslint-disable-next-line react-hooks/rules-of-hooks

    useRouteChanged(router, onRouteChange);
  }

  return route;
}

function useRouterView() {
  return useContext(RouterViewContext);
}

function useMatchedRouteIndex(matchedOffset = 0) {
  const view = useRouterView();
  const index = view ? view.state._routerDepth : 0;
  return Math.max(0, index + matchedOffset);
}

function useMatchedRouteAndIndex(defaultRouter, options = {}) {
  const {
    matchedOffset
  } = options;
  const router = useRouter(defaultRouter);
  const routeIndex = useMatchedRouteIndex(matchedOffset);
  const matchRouteWatch = useCallback((route, prevRoute) => route.matched[routeIndex] !== prevRoute.matched[routeIndex], [routeIndex]);
  const route = useRoute(defaultRouter, options, matchRouteWatch);
  let matched = getRouteMatched(router, route, options.commonPageName);
  const matchedRoute = matched && matched[routeIndex] || null;
  return [matchedRoute, routeIndex];
}

function useMatchedRoute(defaultRouter, options) {
  const ret = useMatchedRouteAndIndex(defaultRouter, options);
  return ret[0];
}

function useRouteMeta(metaKey, defaultRouter, options) {
  const router = useRouter(defaultRouter);
  const route = useMatchedRoute(router, options);
  const meta = route && route.meta;
  const keyIsArray = Array.isArray(metaKey);
  if (keyIsArray && !metaKey.length) throw new Error('metaKey is Empty!');
  const [value, _setValue] = useState(() => keyIsArray ? metaKey.reduce((p, key) => {
    if (!meta) return p;
    p[key] = meta[key];
    return p;
  }, {}) : {
    [metaKey]: meta && meta[metaKey]
  });
  const setValue = useCallback((newValue, setAll = false) => {
    if (!meta) return;

    if (!keyIsArray) {
      newValue = {
        [metaKey]: newValue
      };
    } else if (!setAll) {
      newValue = Object.keys(newValue).reduce((p, key) => {
        if (metaKey.includes(key)) p[key] = newValue[key];
        return p;
      }, {});
    }

    let changed = route && router && router.updateRouteMeta(route, newValue, {
      ignoreConfigRoute: options?.ignoreConfigRoute
    });
    if (!changed) return;

    _setValue(_objectSpread(_objectSpread({}, value), newValue));
  }, [meta, keyIsArray, route, router, value, metaKey, options?.ignoreConfigRoute]);
  return keyIsArray ? [value, setValue] : [value[metaKey], setValue];
}

function useRouteState(defaultRouter, options) {
  const router = useRouter(defaultRouter);
  const route = useMatchedRoute(router || defaultRouter, options);
  const routeUrl = route && route.url;
  const setRouteState = useCallback(newState => router && router.replaceState(newState, route || undefined), // eslint-disable-next-line react-hooks/exhaustive-deps
  [router, routeUrl]);
  return [route ? route.state : {}, setRouteState];
}

function useRouteChanged(router, onChange, deps = []) {
  const [plugin] = useState({});
  plugin.onRouteChange = onChange;
  return useEffect(() => {
    const unplugin = router.plugin(plugin);
    return () => unplugin && unplugin(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [router, plugin, ...deps]);
}

function useRouteMetaChanged(router, onChange, deps = []) {
  const [plugin] = useState({});
  plugin.onRouteMetaChange = useCallback((newVal, oldVal, route, router) => {
    if (deps.length && !deps.some(v => {
      if (isPlainObject(v)) return v === route.meta;
      return v in oldVal;
    })) return;
    return onChange(newVal, oldVal, route, router);
  }, [deps, onChange]);
  return useEffect(() => {
    const unplugin = router.plugin(plugin);
    return () => unplugin && unplugin(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [router, plugin, ...deps]);
}

function createRouteGuardsRef(ref) {
  if (ref && !ref.__routeGuardInfoHooks) innumerable(ref, '__routeGuardInfoHooks', true);
  return ref;
}

function useRouteGuardsRef(ref, guards, deps = []) {
  if (!isFunction(guards)) deps.concat(guards);
  return useImperativeHandle(ref, (...args) => {
    let ret;
    if (isFunction(guards)) ret = guards(...args);else ret = guards;
    return createRouteGuardsRef(ret);
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  deps);
}

export { isCommonPage, getRouteMatched, useRouter, useRouteChanged, useRouteMetaChanged, useRoute, useRouteMeta, useRouteState, useRouterView, useMatchedRouteIndex, useMatchedRoute, useMatchedRouteAndIndex, useRouteGuardsRef, createRouteGuardsRef };