function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { innumerable } from './util';
import { REACT_LAZY_TYPE } from './route-guard';

function isEsModule(value) {
  return value && value.__esModule;
}

export class RouteLazy {
  constructor(ctor, options = {}) {
    _defineProperty(this, "_ctor", void 0);

    _defineProperty(this, "_result", void 0);

    _defineProperty(this, "resolved", void 0);

    _defineProperty(this, "routeLazyInstance", void 0);

    _defineProperty(this, "$$typeof", REACT_LAZY_TYPE);

    _defineProperty(this, "options", void 0);

    _defineProperty(this, "updaters", []);

    this._ctor = ctor;
    this._result = null;
    this.routeLazyInstance = true;
    this.options = options;
    this.render = this.render.bind(this);
    this.resolved = false;
    this.updaters = [];
  }

  toResolve(router, route, key) {
    return new Promise(async (resolve, reject) => {
      if (this.resolved) {
        resolve(this._result);
        return;
      }

      let _resolve = v => {
        v = isEsModule(v) ? v.default : v;
        this.updaters.forEach(updater => v = updater(v) || v);
        this._result = v;
        this.resolved = true;
        resolve(v);
      };

      let component = this._ctor.__lazyImportMethod ? this._ctor(route, key, router, this.options) : this._ctor;

      try {
        component = isPromise(component) ? await component : component;
      } catch (ex) {
        reject(ex);
        return;
      }

      if (!component) {
        reject(new Error('component should not null!'));
        return;
      }

      if (component instanceof Promise) {
        component.then(_resolve).catch(function () {
          return reject(...arguments);
        });
      } else _resolve(component);
    });
  }

  render(props, ref) {
    if (!this.resolved || !this._result) return null;
    return React.createElement(this._result, _objectSpread(_objectSpread({}, props), {}, {
      ref
    }), props.children);
  }

}
export function hasRouteLazy(route) {
  const config = route.config || route;
  if (config.components instanceof RouteLazy) return true;

  if (config.components) {
    for (let key of Object.keys(config.components)) {
      if (config.components[key] instanceof RouteLazy) return true;
    }
  }

  return false;
}
export function hasMatchedRouteLazy(matched) {
  return matched && matched.some(r => hasRouteLazy(r));
}
export function lazyImport(importMethod, options = {}) {
  innumerable(importMethod, '__lazyImportMethod', true);
  return new RouteLazy(importMethod, options || {});
}
export function isRouteLazy(value) {
  return value && value.routeLazyInstance;
}
export function isPromise(value) {
  return value && value.then;
}