const _excluded = ["router", "onRouteActive", "onRouteInactive", "onRouteChange", "exact", "activeClass", "exactActiveClass", "tag", "to", "replace", "append", "event", "children"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import PropTypes from 'prop-types';
import { camelize, normalizeLocation, getHostRouterView, isPropChanged } from './util';

function guardEvent(e) {
  // don't redirect with control keys
  if (e.metaKey || e.altKey || e.ctrlKey || e.shiftKey) return; // don't redirect when preventDefault called

  if (e.defaultPrevented) return; // don't redirect on right click

  if (e.button !== undefined && e.button !== 0) return; // don't redirect if `target="_blank"`

  if (e.currentTarget && e.currentTarget.getAttribute) {
    const target = e.currentTarget.getAttribute('target');
    if (/\b_blank\b/i.test(target)) return;
  } // this may be a Weex event which doesn't have this method


  if (e.preventDefault) {
    e.preventDefault();
  }

  return true;
}

let routerLinkSeed = 0;

class RouterLink extends React.Component {
  constructor(props) {
    super(props);

    _defineProperty(this, "unplugin", void 0);

    const router = props.router;
    this.state = {
      inited: false,
      router,
      routerView: null,
      seed: routerLinkSeed++,
      isMatched: false
    };
  }

  _remount() {
    let {
      router,
      seed
    } = this.state;

    if (this.unplugin) {
      this.unplugin();
      this.unplugin = undefined;
    }

    let routerView = router ? router.viewRoot : getHostRouterView(this);
    if (!router && routerView) router = routerView.state.router;
    this.unplugin = router ? router.plugin({
      name: `router-link-plugin-${seed}`,
      onRouteChange: currentRoute => {
        const {
          isMatched: isMatchedOld
        } = this.state;
        const {
          onRouteChange,
          onRouteActive,
          onRouteInactive
        } = this.props;
        let isMatched = this.isMatched(currentRoute);
        this.setState({
          isMatched
        });

        if (isMatched !== isMatchedOld) {
          if (isMatched) onRouteActive && onRouteActive(currentRoute, this);else onRouteInactive && onRouteInactive(currentRoute, this);
        }

        if (onRouteChange) onRouteChange(currentRoute, this);
      }
    }) : undefined;
    const {
      onRouteActive
    } = this.props;
    const isMatched = Boolean(router && this.isMatched(router.currentRoute, routerView));
    if (router && isMatched) onRouteActive && onRouteActive(router.currentRoute, this);
    const state = {
      inited: true,
      router,
      routerView,
      isMatched
    };
    this.setState(state);
  }

  getFallbackClassName(isMatched) {
    let {
      exact,
      activeClass,
      exactActiveClass
    } = this.props;
    const {
      router
    } = this.state;

    if (router) {
      if (router.linkActiveClass) {
        activeClass = activeClass ? `${activeClass} ${router.linkActiveClass}` : router.linkActiveClass;
      }

      if (router.linkExactActiveClass) {
        exactActiveClass = exactActiveClass ? `${exactActiveClass} ${router.linkExactActiveClass}` : router.linkExactActiveClass;
      }
    }

    let fallbackClass = '';
    if (isMatched) fallbackClass = exact ? exactActiveClass : activeClass;
    return fallbackClass;
  }

  isMatched(currentRoute = null, routerView = null) {
    const router = this.state.router;
    if (!currentRoute && router) currentRoute = router.currentRoute;
    if (!currentRoute) return false;
    let {
      to,
      exact,
      append
    } = this.props;
    if (!routerView) routerView = this.state.routerView;
    to = normalizeLocation(to, {
      route: routerView ? routerView.state.currentRoute : null,
      append,
      queryProps: router && router.queryProps
    });
    let isMatched = false;

    if (to && currentRoute) {
      isMatched = exact ? to.path === currentRoute.path : currentRoute.path.startsWith(to.path);
    }

    return isMatched;
  }

  componentDidMount() {
    this._remount();
  }

  componentWillUnmount() {
    if (this.unplugin) {
      this.unplugin();
      this.unplugin = undefined;
    }
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (isPropChanged(this.props, nextProps)) return true;
    if (this.state.inited !== nextState.inited) return true;
    if (this.state.isMatched !== nextState.isMatched) return true;
    return false;
  }

  componentDidUpdate(prevProps) {
    const newState = {};
    if (this.props.router !== prevProps.router) newState.router = this.props.router; // if (this.props.to !== prevProps.to) {
    //   if (!isPlainObject(this.props.to) || !isPlainObject(prevProps.to)
    //     || isPropChanged(this.props.to, prevProps.to)) {
    //     newState.to = this.props.to;
    //   }
    // }

    if (Object.keys(newState).length) {
      this.setState(newState, () => this._remount());
    }
  }

  render() {
    if (!this.state.inited) return null;
    if (!this.props.tag) return this.props.children;

    let _this$props = this.props,
        {
      // eslint-disable-next-line no-unused-vars
      router: router1,
      onRouteActive,
      onRouteInactive,
      onRouteChange,
      exact,
      activeClass,
      exactActiveClass,
      tag,
      to,
      replace,
      append,
      event,
      children = []
    } = _this$props,
        remainProps = _objectWithoutProperties(_this$props, _excluded);

    const {
      router,
      isMatched,
      routerView
    } = this.state;
    const events = {};
    to = normalizeLocation(to, {
      route: routerView ? routerView.state.currentRoute : null,
      append,
      queryProps: router && router.queryProps
    });
    let fallbackClass = this.getFallbackClassName(isMatched);

    if (fallbackClass) {
      if (remainProps.className) remainProps.className = `${fallbackClass} ${remainProps.className}`;else remainProps.className = fallbackClass;
    }

    if (!Array.isArray(event)) event = event ? [event] : [];
    event.forEach(evt => {
      if (!evt) return;
      const eventName = evt.startsWith('on') ? evt : camelize(`on-${evt}`);

      events[eventName] = e => {
        if (!to || !router || remainProps.disabled) return;
        if (remainProps[eventName] && remainProps[eventName](e, to) === false) return;
        guardEvent(e);
        if (replace) router.replace(to);else router.push(to);
      };
    });
    if (to && tag === 'a') remainProps.href = to.path;
    return React.createElement(tag, _objectSpread(_objectSpread({}, remainProps), events), children);
  }

}

_defineProperty(RouterLink, "propTypes", void 0);

_defineProperty(RouterLink, "defaultProps", void 0);

RouterLink.propTypes = {
  className: PropTypes.string,
  to: PropTypes.oneOfType([PropTypes.string, PropTypes.object]).isRequired,
  replace: PropTypes.bool,
  append: PropTypes.bool,
  tag: PropTypes.oneOfType([PropTypes.string, PropTypes.elementType]),
  activeClass: PropTypes.string,
  exact: PropTypes.bool,
  disabled: PropTypes.bool,
  exactActiveClass: PropTypes.string,
  event: PropTypes.oneOfType([PropTypes.string, PropTypes.arrayOf(PropTypes.string)]),
  onRouteChange: PropTypes.func,
  onRouteActive: PropTypes.func,
  onRouteInactive: PropTypes.func
};
RouterLink.defaultProps = {
  tag: '',
  activeClass: 'router-link-active',
  exactActiveClass: 'exact-active-class',
  event: 'click'
};
export { RouterLink, guardEvent };
export default function createRouterLink(router) {
  return React.forwardRef((props, ref) => React.createElement(RouterLink, _objectSpread(_objectSpread({
    router
  }, props), {}, {
    ref
  }), props.children));
}