"use strict";

require("core-js/modules/es6.symbol.js");

require("core-js/modules/es6.string.iterator.js");

require("core-js/modules/es6.array.iterator.js");

require("core-js/modules/web.dom.iterable.js");

require("core-js/modules/es6.array.from.js");

require("core-js/modules/es6.object.keys.js");

require("core-js/modules/es6.object.get-own-property-descriptor.js");

require("core-js/modules/es7.object.get-own-property-descriptors.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HistoryType = exports.Action = void 0;
exports.createBrowserHistory = createBrowserHistory;
exports.createBrowserHref = createBrowserHref;
exports.createHashHistory = createHashHistory;
exports.createHashHref = createHashHref;
exports.createMemoryHistory = createMemoryHistory;
exports.createPath = createPath;
exports.getBaseHref = getBaseHref;
exports.getPossibleHashType = getPossibleHashType;
exports.parsePath = parsePath;

require("core-js/modules/es6.object.freeze.js");

require("core-js/modules/es6.regexp.search.js");

require("core-js/modules/es6.string.starts-with.js");

require("core-js/modules/es6.array.map.js");

require("core-js/modules/es6.array.filter.js");

require("core-js/modules/es6.object.to-string.js");

require("core-js/modules/es6.regexp.to-string.js");

require("core-js/modules/es6.array.slice.js");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var HistoryType;
/**
 * Actions represent the type of change to a location value.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#action
 */

exports.HistoryType = HistoryType;

(function (HistoryType) {
  HistoryType["hash"] = "hash";
  HistoryType["browser"] = "browser";
  HistoryType["memory"] = "memory";
})(HistoryType || (exports.HistoryType = HistoryType = {}));

var Action;
exports.Action = Action;

(function (Action) {
  Action["Pop"] = "POP";
  Action["Push"] = "PUSH";
  Action["Replace"] = "REPLACE";
})(Action || (exports.Action = Action = {}));

// eslint-disable-next-line no-constant-condition
var readOnly = false
/** __DEV__* */
? function (obj) {
  return Object.freeze(obj);
} : function (obj) {
  return obj;
}; // //////////////////////////////////////////////////////////////////////////////
// BROWSER
// //////////////////////////////////////////////////////////////////////////////

// const BeforeUnloadEventType = 'beforeunload';
var HashChangeEventType = 'hashchange';
var PopStateEventType = 'popstate';

function createBrowserHref(to, hashType) {
  var path = typeof to === 'string' ? to : createPath(to);

  if (hashType != null) {
    var slashChar = path.substr(0, 1);

    if (hashType === 'slash') {
      if (slashChar !== '/') path = '/' + path;
    } else if (hashType === 'noslash') {
      if (slashChar === '/') path = path.substr(1);
    }
  }

  return path;
}
/**
 * Browser history stores the location in regular URLs. This is the standard for
 * most web apps, but it requires some configuration on the server to ensure you
 * serve the same app at multiple URLs.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createbrowserhistory
 */


function createBrowserHistory() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$window = options.window,
      window = _options$window === void 0 ? document.defaultView : _options$window,
      hashType = options.hashType;
  var globalHistory = window.history;

  function getIndexAndLocation() {
    var _window$location = window.location,
        pathname = _window$location.pathname,
        search = _window$location.search,
        hash = _window$location.hash;
    var state = globalHistory.state || {};
    return [state.idx, readOnly({
      pathname: pathname,
      search: search,
      hash: hash,
      state: state.usr || null,
      key: state.key || 'default'
    })];
  }

  var blockedPopTx = null;
  var blockedPopAp = null;
  var blockedPopDc = null;

  function handlePop(e) {
    var _getIndexAndLocation = getIndexAndLocation(),
        _getIndexAndLocation2 = _slicedToArray(_getIndexAndLocation, 2),
        nextIndex = _getIndexAndLocation2[0],
        nextLocation = _getIndexAndLocation2[1];

    if (nextIndex == null) {
      if (index == null || isNaN(index)) index = 0;
      nextIndex = replaceHistoryState(nextLocation, index + 1);
    }

    var delta = index - nextIndex;
    var nextAction = delta > 0 ? Action.Pop : Action.Push;

    if (blockedPopDc) {
      blockedPopDc({
        index: nextIndex,
        location: nextLocation
      });
      blockedPopDc = null;
    } else if (blockedPopAp) {
      if (nextIndex === blockedPopAp.prevIndex) {
        go(blockedPopAp.delta);
        return;
      }

      blockedPopAp.cb && blockedPopAp.cb();
      blockedPopAp = null;
    } else if (blockers.length && delta) {
      var _callback = function _callback(ok, payload) {
        blockedPopTx = null;

        if (ok) {
          applyTx(nextAction, payload);
          return;
        }

        blockedPopAp = {
          action: action,
          index: index,
          location: location,
          prevIndex: nextIndex,
          delta: delta
        };
        go(blockedPopAp.delta);
      };

      blockedPopTx = {
        index: nextIndex,
        action: nextAction,
        location: nextLocation,
        callback: _callback
      };
      blockers.call(blockedPopTx);
    } else {
      applyTx(nextAction);
    }
  }

  window.addEventListener(PopStateEventType, handlePop);
  var action = Action.Push;

  var _getIndexAndLocation3 = getIndexAndLocation(),
      _getIndexAndLocation4 = _slicedToArray(_getIndexAndLocation3, 2),
      index = _getIndexAndLocation4[0],
      location = _getIndexAndLocation4[1];

  var listeners = createEvents();
  var blockers = createEvents();

  if (index == null) {
    index = 0;
    globalHistory.replaceState(_objectSpread(_objectSpread({}, globalHistory.state), {}, {
      idx: index
    }), '');
  }

  var createHref = function createHref(to) {
    return createBrowserHref(to, hashType);
  };

  function getNextLocation(to) {
    var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    return readOnly(_objectSpread(_objectSpread(_objectSpread({}, location), typeof to === 'string' ? parsePath(to) : to), {}, {
      state: state,
      key: createKey()
    }));
  }

  function getHistoryState(nextLocation, index) {
    return {
      usr: nextLocation.state,
      key: nextLocation.key,
      idx: index
    };
  }

  function getHistoryStateAndUrl(nextLocation, index) {
    return [getHistoryState(nextLocation, index), createHref(nextLocation)];
  }

  function allowTx(action, location, callback) {
    return !blockers.length || (blockers.call({
      action: action,
      location: location,
      callback: callback
    }), false);
  }

  function applyTx(nextAction, payload) {
    action = nextAction;

    var _getIndexAndLocation5 = getIndexAndLocation();

    var _getIndexAndLocation6 = _slicedToArray(_getIndexAndLocation5, 2);

    index = _getIndexAndLocation6[0];
    location = _getIndexAndLocation6[1];
    listeners.call({
      action: action,
      index: index,
      location: location
    }, payload);
  }

  function pushHistoryState(location, index) {
    var _getHistoryStateAndUr = getHistoryStateAndUrl(location, index),
        _getHistoryStateAndUr2 = _slicedToArray(_getHistoryStateAndUr, 2),
        historyState = _getHistoryStateAndUr2[0],
        url = _getHistoryStateAndUr2[1]; // TODO: Support forced reloading
    // try...catch because iOS limits us to 100 pushState calls :/


    try {
      globalHistory.pushState(historyState, '', url);
    } catch (error) {
      // They are going to lose state here, but there is no real
      // way to warn them about it since the page will refresh...
      window.location.assign(url);
    }

    return index;
  }

  function replaceHistoryState(location, index) {
    var _getHistoryStateAndUr3 = getHistoryStateAndUrl(location, index),
        _getHistoryStateAndUr4 = _slicedToArray(_getHistoryStateAndUr3, 2),
        historyState = _getHistoryStateAndUr4[0],
        url = _getHistoryStateAndUr4[1]; // TODO: Support forced reloading


    globalHistory.replaceState(historyState, '', url);
    return index;
  }

  function push(to, state) {
    var nextAction = Action.Push;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      var _cb = function _cb(index) {
        pushHistoryState(nextLocation, index + 1);
        applyTx(nextAction, payload);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = function blockedPopDc(_ref) {
          var index = _ref.index;
          return _cb(index);
        };

        go(to.delta);
        return;
      }

      if (blockedPopAp) blockedPopAp.cb = function () {
        return _cb(index);
      };else _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function replace(to, state) {
    var nextAction = Action.Replace;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      var _cb = function _cb(index) {
        replaceHistoryState(nextLocation, index);
        applyTx(nextAction, payload);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = function blockedPopDc(_ref2) {
          var index = _ref2.index;
          return _cb(index);
        };

        go(to.delta);
        return;
      }

      if (blockedPopAp) blockedPopAp.cb = function () {
        return _cb(index);
      };else _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function go(delta) {
    globalHistory.go(delta);
  }

  var history = {
    get extra() {
      return options.extra;
    },

    get type() {
      return HistoryType.browser;
    },

    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    get realtimeLocation() {
      var _getIndexAndLocation7 = getIndexAndLocation(),
          _getIndexAndLocation8 = _slicedToArray(_getIndexAndLocation7, 2),
          current = _getIndexAndLocation8[1];

      return current.pathname === location.pathname && current.search === location.search && current.hash === location.hash ? location : current;
    },

    createHref: createHref,
    getIndexAndLocation: getIndexAndLocation,
    push: push,
    replace: replace,
    replaceState: function replaceState(state) {
      var historyState = getHistoryState(location, index);
      historyState.usr = state;
      globalHistory.replaceState(historyState, '');
      return state;
    },
    go: go,
    back: function back() {
      go(-1);
    },
    forward: function forward() {
      go(1);
    },
    listen: function listen(listener) {
      return listeners.push(listener);
    },
    block: function block(blocker) {
      var unblock = blockers.push(blocker);
      return function () {
        unblock();
      };
    }
  };
  return history;
} // //////////////////////////////////////////////////////////////////////////////
// HASH
// //////////////////////////////////////////////////////////////////////////////


function getPossibleHashType() {
  var locationHash = window.location.hash.substr(1, 1);
  return !locationHash || locationHash === '/' ? 'slash' : 'noslash';
}

function createHashHref(to, hashType) {
  if (!hashType) hashType = getPossibleHashType();
  var path = typeof to === 'string' ? to : createPath(to);
  var slashChar = path.substr(0, 1);

  if (hashType === 'slash') {
    if (slashChar !== '/') path = '/' + path;
  } else if (hashType === 'noslash') {
    if (slashChar === '/') path = path.substr(1);
  }

  if (!path.startsWith('#')) path = '#' + path;
  return getBaseHref() + path;
}
/**
 * Hash history stores the location in window.location.hash. This makes it ideal
 * for situations where you don't want to send the location to the server for
 * some reason, either because you do cannot configure it or the URL space is
 * reserved for something else.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createhashhistory
 */


function createHashHistory() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$window2 = options.window,
      window = _options$window2 === void 0 ? document.defaultView : _options$window2;
  var _options$hashType = options.hashType,
      hashType = _options$hashType === void 0 ? getPossibleHashType() : _options$hashType;
  var globalHistory = window.history;

  function getIndexAndLocation() {
    var locationHash = window.location.hash.substr(1);
    if (locationHash && !locationHash.startsWith('/')) locationHash = '/' + locationHash;

    var _parsePath = parsePath(locationHash),
        _parsePath$pathname = _parsePath.pathname,
        pathname = _parsePath$pathname === void 0 ? '/' : _parsePath$pathname,
        _parsePath$search = _parsePath.search,
        search = _parsePath$search === void 0 ? '' : _parsePath$search,
        _parsePath$hash = _parsePath.hash,
        hash = _parsePath$hash === void 0 ? '' : _parsePath$hash;

    var state = globalHistory.state || {};
    return [state.idx, readOnly({
      pathname: pathname,
      search: search,
      hash: hash,
      state: state.usr || null,
      key: state.key || 'default'
    })];
  }

  var blockedPopTx = null;
  var blockedPopAp = null;
  var blockedPopDc = null;

  function handlePop(e) {
    var _getIndexAndLocation9 = getIndexAndLocation(),
        _getIndexAndLocation10 = _slicedToArray(_getIndexAndLocation9, 2),
        nextIndex = _getIndexAndLocation10[0],
        nextLocation = _getIndexAndLocation10[1];

    if (nextIndex == null) {
      if (index == null || isNaN(index)) index = 0;
      nextIndex = replaceHistoryState(nextLocation, index + 1);
    }

    var delta = index - nextIndex;
    var nextAction = delta > 0 ? Action.Pop : Action.Push;

    if (blockedPopDc) {
      blockedPopDc({
        index: nextIndex,
        location: nextLocation
      });
      blockedPopDc = null;
    } else if (blockedPopAp) {
      if (nextIndex === blockedPopAp.prevIndex) {
        go(blockedPopAp.delta);
        return;
      }

      blockedPopAp.cb && blockedPopAp.cb();
      blockedPopAp = null;
    } else if (blockers.length && delta) {
      var _callback2 = function _callback2(ok, payload) {
        blockedPopTx = null;

        if (ok) {
          applyTx(nextAction, payload);
          return;
        }

        blockedPopAp = {
          action: action,
          index: index,
          location: location,
          prevIndex: nextIndex,
          delta: delta
        };
        go(blockedPopAp.delta);
      };

      blockedPopTx = {
        index: nextIndex,
        action: nextAction,
        location: nextLocation,
        callback: _callback2
      };
      blockers.call(blockedPopTx);
    } else {
      applyTx(nextAction);
    }
  }

  window.addEventListener(PopStateEventType, handlePop); // popstate does not fire on hashchange in IE 11 and old (trident) Edge
  // https://developer.mozilla.org/de/docs/Web/API/Window/popstate_event

  window.addEventListener(HashChangeEventType, function (e) {
    if (blockedPopTx || blockedPopAp) return;

    var _getIndexAndLocation11 = getIndexAndLocation(),
        _getIndexAndLocation12 = _slicedToArray(_getIndexAndLocation11, 2),
        nextLocation = _getIndexAndLocation12[1]; // Ignore extraneous hashchange events.


    if (createPath(nextLocation) !== createPath(location)) {
      handlePop(e);
    }
  });
  var action = Action.Push;

  var _getIndexAndLocation13 = getIndexAndLocation(),
      _getIndexAndLocation14 = _slicedToArray(_getIndexAndLocation13, 2),
      index = _getIndexAndLocation14[0],
      location = _getIndexAndLocation14[1];

  var listeners = createEvents();
  var blockers = createEvents();

  if (index == null) {
    index = 0;
    globalHistory.replaceState(_objectSpread(_objectSpread({}, globalHistory.state), {}, {
      idx: index
    }), '');
  }

  var createHref = function createHref(to) {
    return createHashHref(to, hashType);
  };

  function getNextLocation(to) {
    var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    return readOnly(_objectSpread(_objectSpread(_objectSpread({}, location), typeof to === 'string' ? parsePath(to) : to), {}, {
      state: state,
      key: createKey()
    }));
  }

  function getHistoryState(nextLocation, index) {
    return {
      usr: nextLocation.state,
      key: nextLocation.key,
      idx: index
    };
  }

  function getHistoryStateAndUrl(nextLocation, index) {
    return [getHistoryState(nextLocation, index), createHref(nextLocation)];
  }

  function allowTx(action, location, callback) {
    return !blockers.length || (blockers.call({
      action: action,
      location: location,
      callback: callback
    }), false);
  }

  function applyTx(nextAction, payload) {
    action = nextAction;
    var prevIndex = index;

    var _getIndexAndLocation15 = getIndexAndLocation();

    var _getIndexAndLocation16 = _slicedToArray(_getIndexAndLocation15, 2);

    index = _getIndexAndLocation16[0];
    location = _getIndexAndLocation16[1];

    if (index == null && prevIndex != null) {
      index = nextAction === Action.Push ? prevIndex + 1 : nextAction === Action.Replace ? prevIndex : prevIndex - 1;
    }

    listeners.call({
      action: action,
      index: index,
      location: location
    }, payload);
  }

  function pushHistoryState(location, index) {
    var _getHistoryStateAndUr5 = getHistoryStateAndUrl(location, index),
        _getHistoryStateAndUr6 = _slicedToArray(_getHistoryStateAndUr5, 2),
        historyState = _getHistoryStateAndUr6[0],
        url = _getHistoryStateAndUr6[1]; // TODO: Support forced reloading
    // try...catch because iOS limits us to 100 pushState calls :/


    try {
      globalHistory.pushState(historyState, '', url);
    } catch (error) {
      // They are going to lose state here, but there is no real
      // way to warn them about it since the page will refresh...
      window.location.assign(url);
    }

    return index;
  }

  function replaceHistoryState(location, index) {
    var _getHistoryStateAndUr7 = getHistoryStateAndUrl(location, index),
        _getHistoryStateAndUr8 = _slicedToArray(_getHistoryStateAndUr7, 2),
        historyState = _getHistoryStateAndUr8[0],
        url = _getHistoryStateAndUr8[1]; // TODO: Support forced reloading


    globalHistory.replaceState(historyState, '', url);
    return index;
  }

  function push(to, state) {
    var nextAction = Action.Push;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      var _cb = function _cb(index) {
        pushHistoryState(nextLocation, index + 1);
        applyTx(nextAction, payload);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = function blockedPopDc(_ref3) {
          var index = _ref3.index;
          return _cb(index);
        };

        go(to.delta);
        return;
      }

      if (blockedPopAp) blockedPopAp.cb = function () {
        return _cb(index);
      };else _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true, to);
    }
  }

  function replace(to, state) {
    var nextAction = Action.Replace;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      var _cb = function _cb(index) {
        replaceHistoryState(nextLocation, index);
        applyTx(nextAction, payload);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = function blockedPopDc(_ref4) {
          var index = _ref4.index;
          return _cb(index);
        };

        go(to.delta);
        return;
      }

      if (blockedPopAp) blockedPopAp.cb = function () {
        return _cb(index);
      };else _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function go(delta) {
    globalHistory.go(delta);
  }

  var history = {
    get extra() {
      return options.extra;
    },

    get type() {
      return HistoryType.hash;
    },

    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    get hashType() {
      return hashType;
    },

    get realtimeLocation() {
      var _getIndexAndLocation17 = getIndexAndLocation(),
          _getIndexAndLocation18 = _slicedToArray(_getIndexAndLocation17, 2),
          current = _getIndexAndLocation18[1];

      return current.pathname === location.pathname && current.search === location.search && current.hash === location.hash ? location : current;
    },

    createHref: createHref,
    getIndexAndLocation: getIndexAndLocation,
    push: push,
    replace: replace,
    replaceState: function replaceState(state) {
      var historyState = getHistoryState(location, index);
      historyState.usr = state;
      globalHistory.replaceState(historyState, '');
      return state;
    },
    go: go,
    back: function back() {
      go(-1);
    },
    forward: function forward() {
      go(1);
    },
    listen: function listen(listener) {
      return listeners.push(listener);
    },
    block: function block(blocker) {
      var unblock = blockers.push(blocker);
      return function () {
        unblock();
      };
    }
  };
  return history;
} // //////////////////////////////////////////////////////////////////////////////
// MEMORY
// //////////////////////////////////////////////////////////////////////////////

/**
 * A user-supplied object that describes a location. Used when providing
 * entries to `createMemoryHistory` via its `initialEntries` option.
 */


/**
 * Memory history stores the current location in memory. It is designed for use
 * in stateful non-browser environments like tests and React Native.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#creatememoryhistory
 */
function createMemoryHistory() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$initialEntri = options.initialEntries,
      initialEntries = _options$initialEntri === void 0 ? ['/'] : _options$initialEntri,
      initialIndex = options.initialIndex;
  var entries = initialEntries.map(function (entry) {
    var location = readOnly(_objectSpread({
      pathname: '/',
      search: '',
      hash: '',
      state: null,
      key: createKey()
    }, typeof entry === 'string' ? parsePath(entry) : entry));
    return location;
  });
  var index = clamp(initialIndex == null ? entries.length - 1 : initialIndex, 0, entries.length - 1);
  var action = Action.Push;
  var location = entries[index];
  var listeners = createEvents();
  var blockers = createEvents();

  function createHref(to) {
    return typeof to === 'string' ? to : createPath(to);
  }

  function getNextLocation(to) {
    var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var _location = location,
        _location$pathname = _location.pathname,
        pathname = _location$pathname === void 0 ? '/' : _location$pathname,
        _location$search = _location.search,
        search = _location$search === void 0 ? '' : _location$search,
        _location$hash = _location.hash,
        hash = _location$hash === void 0 ? '' : _location$hash;
    return readOnly(_objectSpread(_objectSpread({
      pathname: pathname,
      search: search,
      hash: hash
    }, typeof to === 'string' ? parsePath(to) : to), {}, {
      state: state,
      key: createKey()
    }));
  }

  function allowTx(action, location, callback) {
    return !blockers.length || (blockers.call({
      action: action,
      location: location,
      callback: callback
    }), false);
  }

  function applyTx(nextAction, nextLocation, nextIndex, payload) {
    action = nextAction;
    location = nextLocation;
    index = nextIndex;
    listeners.call({
      action: action,
      location: location,
      index: index
    }, payload);
  }

  function push(to, state) {
    var nextAction = Action.Push;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      if (typeof to !== 'string' && to.delta) {
        index = clamp(index + to.delta, 0, entries.length - 1);
      }

      index += 1;
      entries.splice(index, entries.length, nextLocation);
      applyTx(nextAction, nextLocation, index, payload);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true, to);
    }
  }

  function replace(to, state) {
    var nextAction = Action.Replace;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      if (typeof to !== 'string' && to.delta) {
        index = clamp(index + to.delta, 0, entries.length - 1);
      }

      entries[index] = nextLocation;
      applyTx(nextAction, nextLocation, index, payload);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true, to);
    }
  }

  function go(delta) {
    var nextIndex = clamp(index + delta, 0, entries.length - 1);
    var nextAction = Action.Pop;
    var nextLocation = entries[nextIndex];

    var callback = function callback(ok, payload) {
      if (!ok) return;
      index = nextIndex;
      applyTx(nextAction, nextLocation, index, payload);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  var history = {
    get extra() {
      return options.extra;
    },

    get type() {
      return HistoryType.memory;
    },

    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    get realtimeLocation() {
      return location;
    },

    createHref: createHref,
    getIndexAndLocation: function getIndexAndLocation() {
      return [index, location];
    },
    push: push,
    replace: replace,
    replaceState: function replaceState(state) {
      return location.state = state;
    },
    go: go,
    back: function back() {
      go(-1);
    },
    forward: function forward() {
      go(1);
    },
    listen: function listen(listener) {
      return listeners.push(listener);
    },
    block: function block(blocker) {
      return blockers.push(blocker);
    }
  };
  return history;
} // //////////////////////////////////////////////////////////////////////////////
// UTILS
// //////////////////////////////////////////////////////////////////////////////


function clamp(n, lowerBound, upperBound) {
  return Math.min(Math.max(n, lowerBound), upperBound);
} // function promptBeforeUnload(event: BeforeUnloadEvent) {
//   // Cancel the event.
//   event.preventDefault();
//   // Chrome (and legacy IE) requires returnValue to be set.
//   event.returnValue = '';
// }


function createEvents() {
  var handlers = [];
  return {
    get length() {
      return handlers.length;
    },

    push: function push(fn) {
      handlers.push(fn);
      return function () {
        handlers = handlers.filter(function (handler) {
          return handler !== fn;
        });
      };
    },
    call: function call(arg, payload) {
      handlers.forEach(function (fn) {
        return fn && fn(arg, payload);
      });
    }
  };
}

function createKey() {
  return Math.random().toString(36).substr(2, 8);
}

function getBaseHref() {
  var base = document.querySelector('base');
  var href = '';

  if (base && base.getAttribute('href')) {
    var url = window.location.href;
    var hashIndex = url.indexOf('#');
    href = hashIndex === -1 ? url : url.slice(0, hashIndex);
  }

  return href;
}
/**
 * Creates a string URL path from the given pathname, search, and hash components.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createpath
 */


function createPath(_ref5) {
  var _ref5$pathname = _ref5.pathname,
      pathname = _ref5$pathname === void 0 ? '/' : _ref5$pathname,
      _ref5$search = _ref5.search,
      search = _ref5$search === void 0 ? '' : _ref5$search,
      _ref5$hash = _ref5.hash,
      hash = _ref5$hash === void 0 ? '' : _ref5$hash;
  return pathname + search + hash;
}
/**
 * Parses a string URL path into its separate pathname, search, and hash components.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#parsepath
 */


function parsePath(path) {
  var partialPath = {};

  if (path) {
    var hashIndex = path.indexOf('#');

    if (hashIndex >= 0) {
      partialPath.hash = path.substr(hashIndex);
      path = path.substr(0, hashIndex);
      if (path && !path.startsWith('/')) path = '/' + path;
    }

    var searchIndex = path.indexOf('?');

    if (searchIndex >= 0) {
      partialPath.search = path.substr(searchIndex);
      path = path.substr(0, searchIndex);
      if (path && !path.startsWith('/')) path = '/' + path;
    }

    if (path) {
      partialPath.pathname = path;
    }
  }

  return partialPath;
}
//# sourceMappingURL=history.js.map