"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createBrowserHistory = createBrowserHistory;
exports.createBrowserHref = createBrowserHref;

var _types = require("./types");

var _utils = require("./utils");

var _history = require("./history");

function createBrowserHref(to, hashType) {
  return (0, _utils.createHref)(to, hashType);
}
/**
 * Browser history stores the location in regular URLs. This is the standard for
 * most web apps, but it requires some configuration on the server to ensure you
 * serve the same app at multiple URLs.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createbrowserhistory
 */


function createBrowserHistory() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$window = options.window,
      window = _options$window === void 0 ? document.defaultView : _options$window;
  var _options$hashType = options.hashType,
      hashType = _options$hashType === void 0 ? (0, _utils.getPossibleHashType)(window) : _options$hashType;
  var type = _types.HistoryType.browser;
  var history = (0, _history.createHistory)({
    window: window,
    type: type,
    getLocationPath: function getLocationPath() {
      return window.location;
    },
    createHref: function createHref(to) {
      return createBrowserHref(to, hashType);
    },
    extra: options.extra
  });
  return history;
}
//# sourceMappingURL=browser.js.map