"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHashHistory = createHashHistory;
exports.createHashHref = createHashHref;

require("core-js/modules/es6.string.starts-with.js");

var _types = require("./types");

var _utils = require("./utils");

var _history = require("./history");

function createHashHref(to, hashType) {
  var path = (0, _utils.createHref)(to, hashType);
  if (!path.startsWith('#')) path = '#' + path;
  return (0, _utils.getBaseHref)() + path;
}
/**
 * Hash history stores the location in window.location.hash. This makes it ideal
 * for situations where you don't want to send the location to the server for
 * some reason, either because you do cannot configure it or the URL space is
 * reserved for something else.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createhashhistory
 */


function createHashHistory() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$window = options.window,
      window = _options$window === void 0 ? document.defaultView : _options$window;
  var _options$hashType = options.hashType,
      hashType = _options$hashType === void 0 ? (0, _utils.getPossibleHashType)(window) : _options$hashType;
  var type = _types.HistoryType.hash;
  var history = (0, _history.createHistory)({
    window: window,
    type: type,
    getLocationPath: function getLocationPath() {
      var locationHash = window.location.hash.substr(1);
      if (locationHash && !locationHash.startsWith('/')) locationHash = '/' + locationHash;
      return (0, _utils.parsePath)(locationHash);
    },
    createHref: function createHref(to) {
      return createHashHref(to, hashType);
    },
    extra: options.extra
  });
  (0, _utils.readonly)(history, 'hashType', function () {
    return hashType;
  });
  return history;
}
//# sourceMappingURL=hash.js.map