"use strict";

require("core-js/modules/es6.object.keys.js");

require("core-js/modules/es6.symbol.js");

require("core-js/modules/es6.array.filter.js");

require("core-js/modules/es6.object.get-own-property-descriptor.js");

require("core-js/modules/es7.object.get-own-property-descriptors.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMemoryHistory = createMemoryHistory;
exports.createMemoryHref = createMemoryHref;

require("core-js/modules/es6.array.map.js");

require("core-js/modules/es6.regexp.search.js");

var _types = require("./types");

var _utils = require("./utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createMemoryHref(to) {
  return typeof to === 'string' ? to : (0, _utils.createPath)(to);
}
/**
  * Memory history stores the current location in memory. It is designed for use
  * in stateful non-browser environments like tests and React Native.
  *
  * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#creatememoryhistory
  */


function createMemoryHistory() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$initialEntri = options.initialEntries,
      initialEntries = _options$initialEntri === void 0 ? ['/'] : _options$initialEntri,
      initialIndex = options.initialIndex;
  var entries = initialEntries.map(function (entry) {
    var location = (0, _utils.readOnly)(_objectSpread({
      pathname: '/',
      search: '',
      hash: '',
      state: null,
      key: (0, _utils.createKey)()
    }, typeof entry === 'string' ? (0, _utils.parsePath)(entry) : entry));
    return location;
  });
  var index = (0, _utils.clamp)(initialIndex == null ? entries.length - 1 : initialIndex, 0, entries.length - 1);
  var type = _types.HistoryType.memory;
  var action = _types.Action.Push;
  var location = entries[index];
  var listeners = (0, _utils.createEvents)();
  var blockers = (0, _utils.createEvents)();

  var createHref = function createHref(to) {
    return createMemoryHref(to);
  };

  function getNextLocation(to) {
    var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var _location = location,
        _location$pathname = _location.pathname,
        pathname = _location$pathname === void 0 ? '/' : _location$pathname,
        _location$search = _location.search,
        search = _location$search === void 0 ? '' : _location$search,
        _location$hash = _location.hash,
        hash = _location$hash === void 0 ? '' : _location$hash;
    return (0, _utils.readOnly)(_objectSpread(_objectSpread({
      pathname: pathname,
      search: search,
      hash: hash
    }, typeof to === 'string' ? (0, _utils.parsePath)(to) : to), {}, {
      state: state,
      key: (0, _utils.createKey)()
    }));
  }

  function applyTx(nextAction, nextLocation, nextIndex, payload) {
    action = nextAction;
    location = nextLocation;
    index = nextIndex;
    listeners.call({
      action: action,
      location: location,
      index: index
    }, payload);
  }

  function getIndex(delta) {
    var ret = index;

    if (delta) {
      index = (0, _utils.clamp)(index + delta, 0, entries.length - 1);
    }

    return ret;
  }

  function push(to, state) {
    var nextAction = _types.Action.Push;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;
      index = getIndex(typeof to !== 'string' ? to.delta : undefined) + 1;
      entries.splice(index, entries.length, nextLocation);
      applyTx(nextAction, nextLocation, index, payload);
    };

    if ((0, _utils.allowTx)(blockers, nextAction, nextLocation, index, getIndex(typeof to !== 'string' ? to.delta : undefined) + 1, callback)) {
      callback(true, to);
    }
  }

  function replace(to, state) {
    var nextAction = _types.Action.Replace;
    var nextLocation = getNextLocation(to, state);

    var callback = function callback(ok, payload) {
      if (!ok) return;

      if (typeof to !== 'string' && to.delta) {
        index = (0, _utils.clamp)(index + to.delta, 0, entries.length - 1);
      }

      entries[index] = nextLocation;
      applyTx(nextAction, nextLocation, index, payload);
    };

    if ((0, _utils.allowTx)(blockers, nextAction, nextLocation, index, getIndex(typeof to !== 'string' ? to.delta : undefined), callback)) {
      callback(true, to);
    }
  }

  function go(delta) {
    var nextIndex = (0, _utils.clamp)(index + delta, 0, entries.length - 1);
    var nextAction = _types.Action.Pop;
    var nextLocation = entries[nextIndex];

    var callback = function callback(ok, payload) {
      if (!ok) return;
      index = nextIndex;
      applyTx(nextAction, nextLocation, index, payload);
    };

    if ((0, _utils.allowTx)(blockers, nextAction, nextLocation, index, getIndex(delta), callback)) {
      callback(true);
    }
  }

  var history = {
    get extra() {
      return options.extra;
    },

    get type() {
      return type;
    },

    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    get length() {
      return entries.length;
    },

    get realtimeLocation() {
      return location;
    },

    createHref: createHref,
    getIndexAndLocation: function getIndexAndLocation() {
      return [index, location];
    },
    push: push,
    replace: replace,
    replaceState: function replaceState(state) {
      return location.state = state;
    },
    go: go,
    back: function back() {
      go(-1);
    },
    forward: function forward() {
      go(1);
    },
    listen: function listen(listener) {
      return listeners.push(listener);
    },
    block: function block(blocker) {
      return blockers.push(blocker);
    }
  };
  return history;
}
//# sourceMappingURL=memory.js.map