"use strict";

require("core-js/modules/es6.object.keys.js");

require("core-js/modules/es6.symbol.js");

require("core-js/modules/es6.object.get-own-property-descriptor.js");

require("core-js/modules/es7.object.get-own-property-descriptors.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.allowTx = allowTx;
exports.clamp = clamp;
exports.createEvents = createEvents;
exports.createHref = createHref;
exports.createKey = createKey;
exports.createPath = createPath;
exports.getBaseHref = getBaseHref;
exports.getPossibleHashType = getPossibleHashType;
exports.parsePath = parsePath;
exports.readOnly = void 0;
exports.readonly = readonly;

require("core-js/modules/es6.object.freeze.js");

require("core-js/modules/es6.array.filter.js");

require("core-js/modules/es6.object.to-string.js");

require("core-js/modules/es6.regexp.to-string.js");

require("core-js/modules/es6.array.slice.js");

require("core-js/modules/es6.regexp.search.js");

require("core-js/modules/es6.string.starts-with.js");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// eslint-disable-next-line no-constant-condition
var readOnly = false
/** __DEV__* */
? function (obj) {
  return Object.freeze(obj);
} : function (obj) {
  return obj;
};
exports.readOnly = readOnly;

function getPossibleHashType() {
  var window = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : document.defaultView;
  var locationHash = window.location.hash.substr(1, 1);
  return !locationHash || locationHash === '/' ? 'slash' : 'noslash';
}

function clamp(n, lowerBound, upperBound) {
  return Math.min(Math.max(n, lowerBound), upperBound);
} // function promptBeforeUnload(event: BeforeUnloadEvent) {
//   // Cancel the event.
//   event.preventDefault();
//   // Chrome (and legacy IE) requires returnValue to be set.
//   event.returnValue = '';
// }


function createEvents() {
  var handlers = [];
  return {
    get length() {
      return handlers.length;
    },

    push: function push(fn) {
      handlers.push(fn);
      return function () {
        handlers = handlers.filter(function (handler) {
          return handler !== fn;
        });
      };
    },
    call: function call(arg, payload) {
      handlers.forEach(function (fn) {
        return fn && fn(arg, payload);
      });
    }
  };
}

function createKey() {
  return Math.random().toString(36).substr(2, 8);
}

function getBaseHref() {
  var base = document.querySelector('base');
  var href = '';

  if (base && base.getAttribute('href')) {
    var url = window.location.href;
    var hashIndex = url.indexOf('#');
    href = hashIndex === -1 ? url : url.slice(0, hashIndex);
  }

  return href;
}
/**
 * Creates a string URL path from the given pathname, search, and hash components.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createpath
 */


function createPath(_ref) {
  var _ref$pathname = _ref.pathname,
      pathname = _ref$pathname === void 0 ? '/' : _ref$pathname,
      _ref$search = _ref.search,
      search = _ref$search === void 0 ? '' : _ref$search,
      _ref$hash = _ref.hash,
      hash = _ref$hash === void 0 ? '' : _ref$hash;
  return pathname + search + hash;
}
/**
 * Parses a string URL path into its separate pathname, search, and hash components.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#parsepath
 */


function parsePath(path) {
  var partialPath = {};

  if (path) {
    var hashIndex = path.indexOf('#');

    if (hashIndex >= 0) {
      partialPath.hash = path.substr(hashIndex);
      path = path.substr(0, hashIndex);
      if (path && !path.startsWith('/')) path = '/' + path;
    }

    var searchIndex = path.indexOf('?');

    if (searchIndex >= 0) {
      partialPath.search = path.substr(searchIndex);
      path = path.substr(0, searchIndex);
      if (path && !path.startsWith('/')) path = '/' + path;
    }

    if (path) {
      partialPath.pathname = path;
    }
  }

  return partialPath;
}

function createHref(to, hashType) {
  if (!hashType) hashType = getPossibleHashType();
  var path = typeof to === 'string' ? to : createPath(to);

  if (hashType != null) {
    var slashChar = path.substr(0, 1);

    if (hashType === 'slash') {
      if (slashChar !== '/') path = '/' + path;
    } else if (hashType === 'noslash') {
      if (slashChar === '/') path = path.substr(1);
    }
  }

  return path;
}

function allowTx(blockers, action, location, index, nextIndex, cb) {
  var resultPayload;
  var count = blockers.length;

  function callback(ok, payload) {
    if (!cb) return;

    if (!ok) {
      cb(ok);
      cb = null;
      return;
    }

    if (arguments.length > 1) resultPayload = payload;

    if (! --count) {
      cb(ok, resultPayload);
      cb = null;
    }
  }

  return !blockers.length || (blockers.call({
    action: action,
    location: location,
    index: index,
    nextIndex: nextIndex,
    callback: callback
  }), false);
}

function readonly(obj, key, get) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
    configurable: true,
    enumerable: true
  };
  Object.defineProperty(obj, key, _objectSpread({
    get: get
  }, options));
  return obj;
}
//# sourceMappingURL=utils.js.map