"use strict";

require("core-js/modules/es6.symbol.js");

require("core-js/modules/es6.string.iterator.js");

require("core-js/modules/es6.object.to-string.js");

require("core-js/modules/es6.array.iterator.js");

require("core-js/modules/web.dom.iterable.js");

require("core-js/modules/es6.array.slice.js");

require("core-js/modules/es6.array.from.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
Object.defineProperty(exports, "isCommonPage", {
  enumerable: true,
  get: function get() {
    return _base.isCommonPage;
  }
});
exports.isTitleRoute = isTitleRoute;
exports.readRouteTitle = readRouteTitle;
exports.walkRouteTitle = walkRouteTitle;

require("core-js/modules/es6.array.filter.js");

require("core-js/modules/es6.array.map.js");

require("core-js/modules/es6.regexp.replace.js");

var _react = require("react");

var _base = require("./base");

var _util = require("../util");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var DEFAULT_TITLE_NAME = 'title';

function readRouteTitle(route) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var ret = {
    visible: false,
    title: ''
  };
  var visible = (0, _util.readRouteMeta)(route, 'visible', options);
  if (visible === false) return ret;
  var titleName = options.titleName || DEFAULT_TITLE_NAME;
  ret.title = (0, _util.readRouteMeta)(route, titleName, options) || '';
  ret.visible = visible !== false && Boolean(ret.title);
  return ret;
}

function walkRouteTitle(router, routes, refresh, filter) {
  var titleName = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : DEFAULT_TITLE_NAME;
  var maxLevel = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 99;
  var level = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : 1;
  routes = (0, _util.getRouteChildren)(routes);
  return routes.filter(function (r) {
    return r.meta.title;
  }).map(function (r) {
    var _readRouteTitle = readRouteTitle(r, {
      titleName: titleName,
      router: router,
      level: level,
      maxLevel: maxLevel,
      refresh: refresh
    }),
        visible = _readRouteTitle.visible,
        title = _readRouteTitle.title;

    if (visible === false) return;
    if (filter && filter(r, routes, {
      title: title,
      visible: visible,
      router: router,
      level: level,
      maxLevel: maxLevel,
      refresh: refresh
    }) === false) return;
    var ret = {
      title: title,
      path: r.path,
      meta: r.meta,
      route: r,
      level: level
    };

    if (level < maxLevel && Array.isArray(r.children) && r.children.length) {
      var children = walkRouteTitle(router, r.children, refresh, filter, titleName, maxLevel, level + 1);
      if (children.length) ret.children = children;
    }

    return ret;
  }).filter(Boolean);
}

function getMatchedDepth(matchedRoute) {
  return matchedRoute ? matchedRoute.depth + 1 : 0;
}

function isTitleRoute(route) {
  var titleName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_TITLE_NAME;
  return route && (0, _util.hasOwnProp)(route.meta, titleName);
}

function walkMatchedRouteList(callback, matched, depth, maxLevel) {
  var titleName = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : DEFAULT_TITLE_NAME;
  if (!callback) return; // let baseLevel = depth;

  while (depth < maxLevel && matched && matched[depth]) {
    var _matchedRouteIndex = depth++;

    var _matchedRoute = matched[_matchedRouteIndex];

    if (isTitleRoute(_matchedRoute, titleName)) {
      // const { visible } = readRouteTitle(matchedRoute.config, titleName, {
      //   maxLevel: maxLevel - baseLevel,
      //   level: depth - baseLevel
      // });
      // if (!visible) break;
      callback(_matchedRoute, _matchedRouteIndex);
    }
  }
}

function getMatchedRouteList(matched, depth, maxLevel, titleName) {
  var ret = [];
  walkMatchedRouteList(function (matchedRoute) {
    ret.push(matchedRoute);
  }, matched, depth, maxLevel, titleName);
  return ret;
}

function getMatchedRoutes(router, matchedRoute, maxLevel, commonPageName, titleName) {
  var currentRoute = router.currentRoute || router.initialRoute;
  var depth = getMatchedDepth(matchedRoute);
  var matchedRoutes = getMatchedRouteList((0, _base.getRouteMatched)(router, currentRoute, commonPageName), depth, maxLevel + depth, titleName);
  return matchedRoutes;
}

function findTitleByMatchedPath(matchedPath, titles, matchedTitles) {
  if (!matchedPath) return undefined;
  var matchedTitle;
  titles.some(function (title) {
    if (title.path === matchedPath) {
      matchedTitle = title;
      if (matchedTitles) matchedTitles.push(title);
      return true;
    }

    if (title.children && title.children.length) {
      matchedTitle = findTitleByMatchedPath(matchedPath, title.children, matchedTitles);
      if (matchedTitle && matchedTitles) matchedTitles.unshift(title);
    }

    return matchedTitle;
  });
  return matchedTitle;
}

function useRouteTitle(props, defaultRouter) {
  var deps = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  if (!props) props = {};
  var _props = props,
      _props$maxLevel = _props.maxLevel,
      maxLevel = _props$maxLevel === void 0 ? 99 : _props$maxLevel,
      _props$filterMetas = _props.filterMetas,
      filterMetas = _props$filterMetas === void 0 ? [] : _props$filterMetas,
      manual = _props.manual,
      _props$matchedOffset = _props.matchedOffset,
      matchedOffset = _props$matchedOffset === void 0 ? 0 : _props$matchedOffset,
      _props$titleName = _props.titleName,
      titleName = _props$titleName === void 0 ? DEFAULT_TITLE_NAME : _props$titleName,
      _props$commonPageName = _props.commonPageName,
      commonPageName = _props$commonPageName === void 0 ? 'commonPage' : _props$commonPageName;

  var _useState = (0, _react.useState)({
    parsed: false,
    mounted: false,
    timerIds: {}
  }),
      _useState2 = _slicedToArray(_useState, 1),
      $refs = _useState2[0];

  $refs.filter = props.filter;
  $refs.onNoMatchedPath = props.onNoMatchedPath;
  var router = (0, _base.useRouter)(defaultRouter);
  if (!router) throw new Error('[useRouteTitle] router can not be null!');

  var _ref = defaultRouter ? [null, 0] // eslint-disable-next-line react-hooks/rules-of-hooks
  : (0, _base.useMatchedRouteAndIndex)(undefined, {
    matchedOffset: matchedOffset,
    commonPageName: commonPageName
  }),
      _ref2 = _slicedToArray(_ref, 2),
      matchedRoute = _ref2[0],
      routeIndex = _ref2[1];

  var _useState3 = (0, _react.useState)(false),
      _useState4 = _slicedToArray(_useState3, 2),
      dirty = _useState4[0],
      setDirty = _useState4[1];

  var refreshTitles = (0, _react.useCallback)(function () {
    return setDirty(true);
  }, [setDirty]);
  var refreshTabs = (0, _react.useCallback)(function () {
    var routes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    $refs.parsed = true;
    return walkRouteTitle(router, routes, refreshTitles, $refs.filter, titleName, maxLevel);
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [router, $refs, maxLevel, refreshTitles].concat(_toConsumableArray(deps)));

  var _useState5 = (0, _react.useState)(function () {
    return manual ? [] : refreshTabs(defaultRouter ? router.routes : matchedRoute ? matchedRoute.config.children : []);
  }),
      _useState6 = _slicedToArray(_useState5, 2),
      titles = _useState6[0],
      setTitles = _useState6[1];

  var _useState7 = (0, _react.useState)(function () {
    return getMatchedRoutes(router, matchedRoute, maxLevel, commonPageName, titleName);
  }),
      _useState8 = _slicedToArray(_useState7, 2),
      matchedRoutes = _useState8[0],
      setMatchedRoutes = _useState8[1];

  var matchedTitles = (0, _react.useMemo)(function () {
    var ret = [];
    if (!matchedRoutes.length || !$refs.parsed) return ret;
    var currentRoute = matchedRoutes[matchedRoutes.length - 1];
    findTitleByMatchedPath(currentRoute.path, titles, ret);
    return ret;
  }, [$refs.parsed, matchedRoutes, titles]);
  var currentPaths = (0, _react.useMemo)(function () {
    if (!matchedRoutes.length || !$refs.parsed) return [];
    var currentRoute = matchedRoutes[matchedRoutes.length - 1];
    var currentTitle = findTitleByMatchedPath(currentRoute.path, titles);

    if (!currentTitle && $refs.mounted && $refs.onNoMatchedPath) {
      if ($refs.timerIds.onNoMatchedPath) clearTimeout($refs.timerIds.onNoMatchedPath);
      $refs.timerIds.onNoMatchedPath = setTimeout(function () {
        $refs.timerIds.onNoMatchedPath = 0;
        if (!$refs.mounted || !$refs.onNoMatchedPath) return;

        var fallback = function fallback(fallbackPath) {
          var toPath = typeof fallbackPath === 'string' ? fallbackPath : fallbackPath.path;
          var currentRoute = router.pendingRoute || router.currentRoute || router.initialRoute;
          if (!toPath || currentRoute && currentRoute.path === toPath) return;
          return router.replace(fallbackPath);
        };

        if (typeof $refs.onNoMatchedPath === 'string') {
          if (!titles.length) return;
          return fallback($refs.onNoMatchedPath === ':first' ? titles[0].path : $refs.onNoMatchedPath);
        }

        $refs.onNoMatchedPath(currentRoute.path, titles, fallback);
      }, 0);
    }

    return currentTitle ? matchedRoutes.map(function (r) {
      return r.path;
    }) : [];
  }, [matchedRoutes, titles, $refs, router]);
  (0, _base.useRouteChanged)(router, (0, _react.useCallback)(function (currentRoute, prevRoute) {
    if (!$refs.mounted) return;
    var routes = getMatchedRoutes(router, matchedRoute, maxLevel, commonPageName, titleName);

    if (routes.length !== matchedRoutes.length || matchedRoutes.some(function (route, i) {
      return route.path !== routes[i].path;
    })) {
      setMatchedRoutes(routes);
    }

    if (!defaultRouter) {
      var matched = (0, _base.getRouteMatched)(router, currentRoute, commonPageName);

      if (matched) {
        var depth = getMatchedDepth(matchedRoute);
        var prevMatched = (0, _base.getRouteMatched)(router, prevRoute, commonPageName);
        var matchedRouteList = getMatchedRouteList(prevMatched, depth, maxLevel, titleName);
        var newMatchedRouteList = getMatchedRouteList(matched, depth, maxLevel, titleName);

        if (matchedRouteList.length !== newMatchedRouteList.length || newMatchedRouteList.some(function (newMatchedRoute, i) {
          return (0, _util.isRouteChanged)(matchedRouteList[i], newMatchedRoute);
        })) {
          var newMatchedRoute = newMatchedRouteList[0];
          setTitles(refreshTabs(newMatchedRoute ? newMatchedRoute.config.parent ? newMatchedRoute.config.parent.children : router.routes : []));
        }
      }
    }
  }, [router, matchedRoute, maxLevel, commonPageName, titleName, matchedRoutes, defaultRouter, refreshTabs, $refs]));
  $refs.routeMetaChangedCallback = (0, _react.useCallback)(function () {
    if (!$refs.mounted) return;
    var matched = (0, _base.getRouteMatched)(router, router.currentRoute, commonPageName);
    var matchedRoute = matched[routeIndex];
    setTitles(refreshTabs(defaultRouter ? router.routes : matchedRoute ? matchedRoute.config.children : []));
  }, [router, commonPageName, routeIndex, refreshTabs, defaultRouter, $refs]);
  (0, _base.useRouteMetaChanged)(router, $refs.routeMetaChangedCallback, [].concat(_toConsumableArray(filterMetas), [titleName, 'visible']));
  (0, _react.useMemo)(function () {
    if (!dirty || !$refs.mounted) return;
    setDirty(false);
    if ($refs.timerIds.dirty) return;
    $refs.timerIds.dirty = setTimeout(function () {
      $refs.timerIds.dirty = 0;
      router.isRunning && $refs.routeMetaChangedCallback && $refs.routeMetaChangedCallback();
    }, 0);
  }, [router, dirty, $refs]);
  (0, _react.useEffect)(function () {
    $refs.mounted = true;
    return function () {
      $refs.mounted = false;
    };
  }, [$refs]);
  return {
    parsed: $refs.parsed,
    titles: titles,
    setTitles: setTitles,
    refreshTitles: refreshTitles,
    matchedRoutes: matchedRoutes,
    matchedTitles: matchedTitles,
    currentPaths: currentPaths
  };
}

var _default = useRouteTitle;
exports.default = _default;
//# sourceMappingURL=use-route-title.js.map