const _excluded = ["transition", "transitionPrefix", "transitionZIndex", "transitionFallback", "routerView", "router", "container", "containerStyle"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { useMemo } from 'react';
import { RouterView as RouterViewOrigin, useRouter, isPlainObject, isFunction } from 'react-view-router';
import SwitchTransition from './SwitchTransition';
import CSSTransition from './CSSTransition';
import './router-view.css';

function isPush(to) {
  return to.action === 'PUSH' || to.params.isPush || to.query.isPush;
}

function isPop(to) {
  return to.action === 'POP' || to.params.isBack || to.params.isPop || to.query.back;
}

function isReplace(to) {
  return to.action === 'REPLACE' && !isPush(to) && !isPop(to);
}
/**
 * @typedef {import('react-view-router').RouterView} RouterView
 */


const RouterViewTransition = React.forwardRef(
/**
 *
 * @param {import('../types/router-view').TransitionRouterViewProps} props
 * @param {React.Ref<RouterView>} ref
 * @returns
 */
(props, ref) => {
  let {
    transition = 'slide',
    transitionPrefix = 'react-view-router-',
    transitionZIndex = 1000,
    transitionFallback = '',
    routerView,
    router: defaultRouter,
    container,
    containerStyle = {}
  } = props,
      restProps = _objectWithoutProperties(props, _excluded);

  const router = useRouter(defaultRouter);
  if (routerView === RouterViewTransition) routerView = null;
  const backgroundColor = useMemo(() => containerStyle && containerStyle.backgroundColor || document.defaultView.getComputedStyle(document.body, null).getPropertyValue('background-color'), // eslint-disable-next-line react-hooks/exhaustive-deps
  [containerStyle && containerStyle.backgroundColor]);
  containerStyle = _objectSpread({
    height: '100%'
  }, containerStyle);
  let containerTag = 'div';

  if (isPlainObject(transition)) {
    transitionZIndex = transition.zIndex || transitionZIndex;
    containerStyle = Object.assign(containerStyle, transition.containerStyle);
    containerTag = transition.containerTag || containerTag;
    transition = transition.name || 'slide';
  }

  const transitionMap = useMemo(() => {
    const isSlideNode = node => node && node.className.includes(transitionPrefix + 'slide');

    const isFadeNode = node => node && node.className.includes(transitionPrefix + 'fade');

    const transitionProps = {
      addEndListener: (node, done) => {
        const to = router.currentRoute;

        if (!isFadeNode(node) && isReplace(to) || node.className.includes('slide-right-enter') || node.className.includes('slide-left-exit')) {
          return done();
        }

        node.addEventListener('transitionend', done, false);
      },
      onEnter: maybeNode => {
        if (isSlideNode(maybeNode)) {
          if (transitionZIndex) maybeNode.style.zIndex = transitionZIndex;
          if (backgroundColor) maybeNode.style.backgroundColor = backgroundColor;
        }
      },
      onEntered: maybeNode => {
        if (isSlideNode(maybeNode)) {
          if (transitionZIndex) maybeNode.style.zIndex = '';
          if (backgroundColor) maybeNode.style.backgroundColor = '';
        }
      },
      onExit: maybeNode => {
        if (isSlideNode(maybeNode)) {
          if (transitionZIndex) maybeNode.style.zIndex = transitionZIndex;
          if (backgroundColor) maybeNode.style.backgroundColor = backgroundColor;
        }
      },
      onExited: maybeNode => {
        if (isSlideNode(maybeNode)) {
          if (transitionZIndex) maybeNode.style.zIndex = '';
          if (backgroundColor) maybeNode.style.backgroundColor = '';
        }
      }
    };
    /**
     * @type {{
     *   [key: string]: {
     *     mode: string,
     *     props: Partial<any>
     * }}
     * */

    const map = {
      fade: {
        mode: 'out-in',
        props: {
          classNames: transitionPrefix + 'fade'
        }
      },
      slide: {
        mode: 'in-out',
        props: {
          classNames: () => {
            let ret = '';
            const to = router.currentRoute;

            if (isPush(to)) {
              ret = 'slide-left';
            } else if (isPop(to)) {
              ret = 'slide-right';
            } else {
              ret = isFunction(transitionFallback) ? transitionFallback(to) : transitionFallback;
            }

            return ret ? `${transitionPrefix}${ret}` : ret;
          }
        }
      },
      carousel: {
        mode: 'together',
        props: {
          classNames: () => {
            let ret = '';
            const to = router.currentRoute;

            if (isPush(to)) {
              ret = 'carousel-left';
            } else if (isPop(to)) {
              ret = 'carousel-right';
            } else {
              ret = isFunction(transitionFallback) ? transitionFallback(to) : transitionFallback;
            }

            return ret ? `${transitionPrefix}${ret}` : ret;
          }
        }
      }
    };
    const transitionItem = map[transition];
    if (transitionItem) Object.assign(transitionItem.props, transitionProps);
    return transitionItem || {
      mode: '',
      props: {}
    };
  }, [router, transitionFallback, transition, transitionPrefix, transitionZIndex, backgroundColor]);
  return React.createElement(routerView || RouterViewOrigin, _objectSpread({
    ref,
    router: defaultRouter,
    container: (result, route) => {
      if (container) result = container(result, route); // console.log('router-view container', route, route.path, this.transitionName, router.stacks.length);

      return React.createElement(SwitchTransition, {
        mode: transitionMap.mode
      }, React.createElement(CSSTransition, _objectSpread({
        key: route.path
      }, transitionMap.props), React.createElement(containerTag, {
        style: containerStyle
      }, result)));
    }
  }, restProps));
});
export default RouterViewTransition;